'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PatternMapping = exports.Collection = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.create = create;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require('path');

var _pathIsAbsolute = require('path-is-absolute');

var _pathIsAbsolute2 = _interopRequireDefault(_pathIsAbsolute);

var _skypagerUtil = require('skypager-util');

var _document = require('../document');

var _document2 = _interopRequireDefault(_document);

var _importer = require('../importer');

var _importer2 = _interopRequireDefault(_importer);

var _scope = require('../scope');

var _scope2 = _interopRequireDefault(_scope);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var string = _lodash2.default.string;
var set = _lodash2.default.set;
var groupBy = _lodash2.default.groupBy;
var snakeCase = _lodash2.default.snakeCase;
var camelCase = _lodash2.default.camelCase;
var uniq = _lodash2.default.uniq;
var _get = _lodash2.default.get;
var _at = _lodash2.default.at;
var _result = _lodash2.default.result;
var chain = _lodash2.default.chain;
var mapValues = _lodash2.default.mapValues;
var _Object = Object;
var values = _Object.values;
var keys = _Object.keys;
var defineProperty = _Object.defineProperty;
var assign = _Object.assign;

var Collection = exports.Collection = (function () {

  /**
   * Create a collection which wraps a directory and creates a queryable index
   * of all of the files inside of it.  Files exist as VFile (@see https://github.com/wooorm/vfile) and are
   * wrapped by Document classes which provide the interface for metadata, transformations, and things of that nature.
   * A collection also provides a lodash wrapper interface for building extraction functions from the collections
   * files and data.
   *
   * @param {Object} options - the options hash
   * @param {String} options.type - the type of document - will determine which document model wrapper is used
   * @param {String} options.base - the root of the collection, defaults to process.cwd()
   * @param {String} options.cwd - the cwd, if different from the base
   * @param {Array} options.include - an array of glob patterns to include
   * @param {Array} options.exclude - an array of glob patterns to exclude
   * @param {Object} options.documentOptions - an object that will be passed to the document wrapper when created
   */

  function Collection() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    (0, _classCallCheck3.default)(this, Collection);

    (0, _skypagerUtil.mixinPropertyUtils)(this);

    this.hide('context', options.context || options.root || options.base || options.basePath);
    this.hide('options', normalize(options));
    this.hide('files', [], true);

    this.pathId = this.context.split('/').reverse().slice(0, 2).join('-');
    this.instanceId = this.pathId + '-' + Math.floor(Date.now() / 1000);

    this.importer = _importer2.default.create(this.context, (0, _extends3.default)({
      patterns: this.patterns
    }, options.importerOptions || {}));

    this.hide('cache', {});
    this.hide('_scopes', {});
  }

  (0, _createClass3.default)(Collection, [{
    key: 'loadFiles',
    value: function loadFiles() {
      var _files;

      var files = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

      this.files.length = 0;
      (_files = this.files).push.apply(_files, (0, _toConsumableArray3.default)(files));
    }
  }, {
    key: 'runImporter',
    value: function runImporter() {
      var _this = this;

      var cacheDocuments = arguments.length <= 0 || arguments[0] === undefined ? true : arguments[0];

      return this.importer.readFiles(false).then(function (files) {
        var _files2;

        return (_files2 = _this.files).push.apply(_files2, (0, _toConsumableArray3.default)(files.map(function (file) {
          return file.toVFile ? file.toVFile() : file;
        })));
      }).then(function () {
        return cacheDocuments ? _this.cached : _this;
      }).catch(function (error) {
        return error;
      });
    }
  }, {
    key: 'getFileByRelativePath',
    value: function getFileByRelativePath(relativeFilePath) {
      var index = this.filesIndex[relativeFilePath].index;

      return this.files[index];
    }
  }, {
    key: 'createRouter',
    value: function createRouter(name) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (options.getter) {
        options.pathsGetter = options.getter;
      }
      if (options.property) {
        options.pathProperty = options.property;
      }

      (0, _skypagerUtil.router)(this._routers, (0, _extends3.default)({
        routerProperty: name,
        pathsGetter: name,
        pathProperty: 'id'
      }, options, {
        host: this,
        matchesOnly: true
      }));
    }
  }, {
    key: 'createDelegators',
    value: function createDelegators(receiver) {
      var _this2 = this;

      for (var _len = arguments.length, propNames = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        propNames[_key - 1] = arguments[_key];
      }

      propNames.forEach(function (propName) {
        receiver.hideGetter(propName, function () {
          return _this2[propName];
        });
      });
    }
  }, {
    key: 'join',
    value: function join() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      return _path.join.apply(undefined, [this.context].concat(args));
    }
  }, {
    key: 'relative',
    value: function relative() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      return _path.relative.apply(undefined, [this.context].concat(args));
    }
  }, {
    key: 'resolve',
    value: function resolve() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      return _path.resolve.apply(undefined, [this.context].concat(args));
    }
  }, {
    key: 'get',
    value: function get() {
      for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        args[_key5] = arguments[_key5];
      }

      return _get.apply(undefined, [this].concat(args));
    }
  }, {
    key: 'result',
    value: function result() {
      for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        args[_key6] = arguments[_key6];
      }

      return _result.apply(undefined, [this].concat(args));
    }
  }, {
    key: 'at',
    value: function at() {
      for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        args[_key7] = arguments[_key7];
      }

      return _at.apply(undefined, [this].concat(args));
    }
  }, {
    key: 'select',
    value: function select() {
      return this.selectChain.apply(this, arguments).value();
    }
  }, {
    key: 'selectChain',
    value: function selectChain() {
      var _chain;

      for (var _len8 = arguments.length, paths = Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        paths[_key8] = arguments[_key8];
      }

      var set = function set(obj, path, val) {
        _lodash2.default.set(obj, path, val);
        return obj;
      };

      return (_chain = this.chain).at.apply(_chain, (0, _toConsumableArray3.default)(_lodash2.default.sortBy(paths, function (p) {
        return p.length;
      }))).map(function (value, index) {
        return [paths[index], value];
      }).reduce(function (memo, pair) {
        return set.apply(undefined, [memo].concat((0, _toConsumableArray3.default)(pair)));
      }, {});
    }
  }, {
    key: 'selectByObject',
    value: function selectByObject(pathObject) {
      var _chain2;

      var keys = keys(pathObject);
      var paths = values(pathObject);

      return (_chain2 = this.chain).at.apply(_chain2, (0, _toConsumableArray3.default)(paths)).reduce(function (memo, val, index) {
        return set(memo, keys[index], val);
      }, {}).value();
    }
  }, {
    key: 'findDocumentById',
    value: function findDocumentById(documentId) {
      return this.documents.find(function (doc) {
        return doc.id === documentId;
      });
    }
  }, {
    key: 'applyFilesRoute',
    value: function applyFilesRoute(pattern) {
      var path = route()(pattern);
      var results = this.files.map(function (f) {
        return { path: f.relative, pathData: path(f.relative) };
      }).filter(function (f) {
        return f.pathData;
      });

      return {
        results: results,
        pattern: pattern
      };
    }
  }, {
    key: 'defineScope',
    value: function defineScope(scopeName, routePattern, options) {
      var name = camelCase(scopeName);

      if (this._scopes[name] && !routePattern && !options) {
        return this._scopes[name];
      }

      return this._scopes[name] = new _scope2.default((0, _extends3.default)({
        pattern: typeof routePattern === 'string' ? new RegExp(routePattern) : routePattern,
        key: scopeName,
        name: name
      }, options || {}));
    }
  }, {
    key: 'type',
    get: function get() {
      return snakeCase(this.options.type || 'Project').replace(/s$/i, '');
    }
  }, {
    key: 'patterns',
    get: function get() {
      return _lodash2.default.uniq(this.includePatterns.concat(this.excludePatterns));
    }
  }, {
    key: 'includePatterns',
    get: function get() {
      return _lodash2.default.uniq(this.options.include.concat(this.typePatterns));
    }
  }, {
    key: 'typePatterns',
    get: function get() {
      switch (this.type.toUpperCase()) {
        case PACKAGE:
          return PatternMapping.Package;
          break;
        case SCRIPT:
          return PatternMapping.Script;
          break;
        case (SETTINGS_FILE, COPY_FILE):
          return PatternMapping.SettingsFile;
          break;
        case DOCUMENT:
          return PatternMapping.Document;
          break;
        case STYLESHEET:
          return PatternMapping.Stylesheet;
          break;
        case VECTOR:
          return PatternMapping.Vector;
          break;
        case PROJECT:
        default:
          return PatternMapping.Project;
      }
    }
  }, {
    key: 'router',
    get: function get() {
      return this.routers.documents;
    }
  }, {
    key: 'routers',
    get: function get() {
      if (!this._routers) {
        this.hide('_routers', {});

        this.createRouter('files', {
          pathProperty: 'relative'
        });

        this.createRouter('documents', {
          pathProperty: 'id',
          pathsGetter: 'documents'
        });

        this.createRouter('folders', {
          pathProperty: 'toString',
          pathsGetter: 'subFolderNames'
        });
      }

      return mapValues(this._routers, function (router) {
        return router.get.bind(router);
      });
    }
  }, {
    key: 'chain',
    get: function get() {
      return chain(this);
    }
  }, {
    key: 'filesTree',
    get: function get() {
      if (this._filesTree) {
        return this._filesTree;
      }

      var tree = this.freshFilesTree;

      this.hide('_filesTree', tree);

      return tree;
    }
  }, {
    key: 'freshFilesTree',
    get: function get() {
      var _this3 = this;

      var base = this.subFolderObjectPaths.reduce(function (memo, objectPath) {
        return set(memo, objectPath.split('.'), {});
      }, {});

      var filePaths = this.relativeObjectPaths;

      this.subFolderObjectPaths.forEach(function (context) {
        var files = _get(filePaths, context, []);

        files.forEach(function (d) {
          var baseName = d[0];
          var rel = d[1];
          set(base, context.split('.').concat(baseName), _this3.getFileByRelativePath(rel));
        });
      });

      return base;
    }
  }, {
    key: 'relativeObjectPaths',
    get: function get() {
      var _this4 = this;

      return this.chain.get('files').sortBy(function (f) {
        return f.dirname.length;
      }).map(function (f) {
        return [_this4.relative(f.dirname).replace(/\//g, '.'), f.basename, f.relative];
      }).groupBy(function (i) {
        return i.shift();
      }).value();
    }
  }, {
    key: 'subFolderObjectPaths',
    get: function get() {
      return this.chain.get('subFolderNames').sortBy(function (v) {
        return v.length;
      }).map(function (p) {
        return p.replace(/\//g, '.');
      }).value();
    }
  }, {
    key: 'scopes',
    get: function get() {
      var collection = this;
      var base = {
        available: keys(collection._scopes)
      };

      base.available.forEach(function (scopeName) {
        return defineProperty(base, scopeName, {
          configurable: false,
          false: true,
          get: function get() {
            var cfg = collection.scope(scopeName);
            return collection.files.filter(function (file) {
              return file.relative.match(cfg.pattern);
            });
          }
        });
      });

      return base;
    }
  }, {
    key: 'excludePatterns',
    get: function get() {
      return this.options.exclude.map(function (p) {
        return p.startsWith('!') ? p : '!' + p;
      }).concat(['!**/node_modules/**', '!_*']);
    }
  }, {
    key: 'imported',
    get: function get() {
      if (!this.filesLoaded) {
        this.loadFiles(this.importer.readFilesSync(false));
      }

      return this.cached;
    }
  }, {
    key: 'rootFiles',
    get: function get() {
      return this.chains.files.filter(function (f) {
        return f.dirname === project.root;
      }).value();
    }
  }, {
    key: 'groupedByChildFolder',
    get: function get() {
      var _this5 = this;

      return this.chains.files.groupBy(function (f) {
        return _this5.relative(f.dirname).split('/').shift();
      }).pickBy(function (v, k) {
        return k.length > 0;
      }).value();
    }
  }, {
    key: 'groupedByChildFolderAndExtension',
    get: function get() {
      var _this6 = this;

      return this.chains.files.groupBy(function (f) {
        return _this6.relative(f.dirname).split('/').shift();
      }).mapValues(function (v) {
        return groupBy(v, function (v) {
          return v.extname;
        });
      }).pickBy(function (v, k) {
        return k.length > 0;
      }).value();
    }
  }, {
    key: 'chains',
    get: function get() {
      return {
        documents: chain(this.documents),
        files: chain(this.files)
      };
    }
  }, {
    key: 'query',
    get: function get() {
      var collection = this;

      return {
        get files() {
          return _skypagerUtil.query.bind(_skypagerUtil.query, collection.files);
        },
        get documents() {
          return _skypagerUtil.query.bind(_skypagerUtil.query, collection.documents);
        }
      };
    }
  }, {
    key: 'depths',
    get: function get() {
      var _this7 = this;

      return this.chains.files.map(function (f) {
        return _this7.relative(f.dirname);
      }).uniq().groupBy(function (f) {
        return f.split(_path.sep).length - 1;
      }).omit('').value();
    }
  }, {
    key: 'requireContexts',
    get: function get() {
      return chain(this.groupedByChildFolder).mapValues(function (files, childFolder) {
        return ['./' + childFolder, true, '.(' + chain(files).map(function (file) {
          return file.extname.replace(/^\./, '');
        }).uniq().value().join('|') + ')$'];
      }).value();
    }
  }, {
    key: 'relativeFilePaths',
    get: function get() {
      var _this8 = this;

      return this.files.map(function (file) {
        return (0, _path.relative)(_this8.context, file.path);
      });
    }
  }, {
    key: 'relativeSubFolderPaths',
    get: function get() {
      var _this9 = this;

      return this.files.map(function (file) {
        return (0, _path.relative)(_this9.context, file.dirname);
      });
    }
  }, {
    key: 'subFolderPaths',
    get: function get() {
      return uniq(this.files.map(function (f) {
        return f.dirname;
      }));
    }
  }, {
    key: 'subFolderNames',
    get: function get() {
      var _this10 = this;

      return this.subFolderPaths.map(function (f) {
        return (0, _path.relative)(_this10.context, f);
      }).filter(function (f) {
        return f.length > 0;
      });
    }
  }, {
    key: 'subFolderBasenames',
    get: function get() {
      var _this11 = this;

      return this.subFolderPaths.map(function (f) {
        return (0, _path.relative)(_this11.context, f);
      }).map(function (f) {
        return (0, _path.basename)(f);
      });
    }
  }, {
    key: 'childFolderNames',
    get: function get() {
      return uniq(this.subFolderNames.map(function (p) {
        return p.split('/').shift();
      }));
    }
  }, {
    key: 'childFolderPaths',
    get: function get() {
      var _this12 = this;

      return uniq(this.childFolderNames.map(function (p) {
        return _this12.join(p);
      }));
    }
  }, {
    key: 'filesIndex',
    get: function get() {
      if (this._filesIndex) {
        return this._filesIndex;
      }
      this.hide('_filesIndex', this.freshFilesIndex);

      return this._filesIndex;
    }
  }, {
    key: 'freshFilesIndex',
    get: function get() {
      return this.chain.get('files').map(function (f, i) {
        return { index: i, path: f.path, relative: f.relative };
      }).keyBy(function (f) {
        return f.relative;
      }).value();
    }
  }, {
    key: 'scope',
    get: function get() {
      var fn = this.defineScope.bind(this);

      return assign(fn, this.scopes);

      return fn;
    }
  }, {
    key: 'ready',
    get: function get() {
      return this.importer.status === 'LOADED' ? _promise2.default.resolve(this) : this.runImporter().then(function (importer) {
        return importer;
      });
    }
  }, {
    key: 'documentOptions',
    get: function get() {
      return (0, _extends3.default)({
        context: this.context,
        collectionType: this.options.type,
        cwd: this.options.cwd || this.context
      }, this.options.documentOptions || {});
    }
  }, {
    key: 'cached',
    get: function get() {
      var _this13 = this;

      this.files.forEach(function (file) {
        _this13.cache[file.path] = _this13.cache[file.path] || new _document2.default(file, _this13.documentOptions);
      });

      return this;
    }
  }, {
    key: 'documents',
    get: function get() {
      return values(this.cache);
    }
  }], [{
    key: 'mount',
    value: function mount() {
      var base = arguments.length <= 0 || arguments[0] === undefined ? process.cwd() : arguments[0];
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (!(0, _pathIsAbsolute2.default)(base)) {
        base = (0, _path.resolve)(process.cwd(), base);
      }

      return create((0, _extends3.default)({
        base: base
      }, options)).imported.cached;
    }
  }, {
    key: 'create',
    value: function create() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      return new Collection(options);
    }
  }, {
    key: 'Document',
    get: function get() {
      return _document2.default;
    }
  }, {
    key: 'stringUtils',
    get: function get() {
      return require('lodash/string');
    }
  }]);
  return Collection;
})();

exports.default = Collection;

var PACKAGE = 'PACKAGE';
var SCRIPT = 'SCRIPT';
var DOCUMENT = 'DOCUMENT';
var SETTINGS_FILE = 'SETTINGS_FILE';
var COPY_FILE = 'COPY_FILE';
var PROJECT = 'PROJECT';
var VECTOR = 'VECTOR';
var STYLESHEET = 'STYLESHEET';

var PatternMapping = exports.PatternMapping = {
  'Package': ['**/package.json'],
  'Document': ['**/*.md'],
  'Script': ['**/*.js', '**/*.jsx'],
  'Stylesheet': ['**/*.{css,less,sass,scss,styl'],
  'SettingsFile': ['**/*.yml'],
  'CopyFile': ['**/*.yml'],
  'Vector': ['**/*.svg'],
  'DataSource': ['**/*.json', '**/*.yml', '**/*.data.js'],
  'Project': ['**/*']
};

function create() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  return Collection.create(options);
}

function normalize() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  return (0, _extends3.default)({}, options, {
    include: [].concat((0, _toConsumableArray3.default)(options.include || []), (0, _toConsumableArray3.default)(options.patterns || [])),
    exclude: [].concat((0, _toConsumableArray3.default)(options.exclude || []), (0, _toConsumableArray3.default)(options.ignore || []))
  });
}