import isArray from 'lodash/isArray'
import vRead from 'vinyl-read'
import convertVinylToVFile from 'convert-vinyl-to-vfile'
import { mixinPropertyUtils } from 'skypager-util'
import { DefaultPatterns } from '../document'

const { defineProperty } = Object

export const DefaultOptions = {
  patterns: DefaultPatterns,
}

export const toVFile = (vinyl) => convertVinylToVFile(vinyl)

export default class DocumentImporter {
  static patterns = DefaultPatterns;

  static create (context, options = {}) {
    if (typeof options === 'string') {
      options = {
        patterns: [options],
        context
      }
    }

    if (isArray(options)) {
      options = {
        patterns: options,
        context
      }
    }

    const importer = new DocumentImporter(context, {
      context,
      ...options
    })

    return importer
  }

  constructor (context = process.cwd(), options = {}) {
    mixinPropertyUtils(this)

    if (typeof context !== 'string') {
      throw('Document Importers must be created with a context path')
    }

    this.context = context

    this.hide('options', {
      ...DefaultOptions,
      ...options,
    })

    this.hide('patterns', options.patterns || [])

    let status = 'CREATED'

    defineProperty(this, 'status', {
      configurable: false,
      get: () => status,
      set: ((newValue) => status = newValue)
    })
  }

  get fileWrappersSync() {
    return vRead.sync(this.patterns, {
      base: this.context,
      read: this.options.includeContents === true,
      ...this.options
    })
  }

  get fileWrappers() {
    return vRead(this.patterns, {
      base: this.context,
      read: includeContents !== false && this.options.includeContents === true,
      ...this.options
    })
  }

  readFilesSync (includeContents) {
    return vRead.sync(this.patterns, {
      base: this.context,
      read: includeContents !== false && this.options.includeContents === true,
      ...this.options
    })
  }

  async readFiles (includeContents = false) {
    const _files = await vRead(this.patterns, {
      base: this.context,
      read: includeContents === true,
      ...this.options
    })

    return _files
  }
}
